@@ -79,6 +79,7 @@ class SizeInfo(NamedTuple):
 
 class MaskedCond(NamedTuple):
     image_res: ReadImageResponse
+    mask_res: ReadImageResponse
     mask: np.ndarray
     remained_image: np.ndarray
     remained_mask: np.ndarray
@@ -138,6 +139,8 @@ class DiffusionAPI(APIMixin):
                 self.first_stage.half()
             if unconditional_cond is not None:
                 unconditional_cond = unconditional_cond.half()
+            for k, v in self._uncond_cache.items():
+                self._uncond_cache[k] = v.half()
         else:
             if self.cond_model is not None:
                 self.cond_model.float()
@@ -145,12 +148,16 @@ class DiffusionAPI(APIMixin):
                 self.first_stage.float()
             if unconditional_cond is not None:
                 unconditional_cond = unconditional_cond.float()
+            for k, v in self._uncond_cache.items():
+                self._uncond_cache[k] = v.float()
         if self.cond_model is not None:
             self.cond_model.to(device)
         if self.first_stage is not None:
             self.first_stage.to(device)
         if unconditional_cond is not None:
             self.sampler.unconditional_cond = unconditional_cond.to(device)
+        for k, v in self._uncond_cache.items():
+            self._uncond_cache[k] = v.to(device)
 
     @property
     def size_info(self) -> SizeInfo:
@@ -293,7 +300,9 @@ class DiffusionAPI(APIMixin):
                         i_z = repeat(z)
                     else:
                         in_channels = self.m.in_channels
-                        if cond_concat is not None:
+                        if self.cond_type == CONCAT_TYPE:
+                            in_channels -= cond.shape[1]
+                        elif cond_concat is not None:
                             in_channels -= cond_concat.shape[1]
                         i_z_shape = len(i_cond), in_channels, *size[::-1]
                         i_z, _ = self._set_seed_and_variations(
@@ -443,7 +452,8 @@ class DiffusionAPI(APIMixin):
     ) -> MaskedCond:
         # handle mask stuffs
         image_res = read_image(image, max_wh, anchor=anchor)
-        mask = read_image(mask, max_wh, anchor=anchor, to_mask=True).image
+        mask_res = read_image(mask, max_wh, anchor=anchor, to_mask=True)
+        mask = mask_res.image
         bool_mask = mask >= 0.5
         remained_mask = (~bool_mask).astype(np.float16 if self.use_half else np.float32)
         remained_image = mask_image_fn(remained_mask, image_res.image)
@@ -456,6 +466,7 @@ class DiffusionAPI(APIMixin):
         mask_cond = F.interpolate(mask_cond, size=latent_shape)
         return MaskedCond(
             image_res,
+            mask_res,
             mask,
             remained_image,
             remained_mask,
@@ -506,6 +517,7 @@ class DiffusionAPI(APIMixin):
         max_wh: int = 512,
         num_steps: Optional[int] = None,
         clip_output: bool = True,
+        keep_original: bool = False,
         use_raw_inpainting: bool = False,
         raw_inpainting_fidelity: float = 0.2,
         callback: Optional[Callable[[Tensor], Tensor]] = None,
@@ -542,14 +554,16 @@ class DiffusionAPI(APIMixin):
             max_wh,
             anchor,
             lambda remained_mask, img: np.where(remained_mask, img, 0.5),
-            lambda arr: torch.from_numpy(arr),
+            lambda bool_mask: torch.from_numpy(bool_mask),
         )
         # sampling
-        factor = self.size_info.factor
         size = tuple(
-            map(lambda n: n * factor, res.remained_image_cond.shape[-2:][::-1])
+            map(
+                lambda n: n * self.size_info.factor,
+                res.remained_image_cond.shape[-2:][::-1],
+            )
         )
-        return self.sample(
+        sampled = self.sample(
             num_samples,
             export_path,
             size=size,  # type: ignore
@@ -563,6 +577,14 @@ class DiffusionAPI(APIMixin):
             verbose=verbose,
             **kwargs,
         )
+        if keep_original:
+            original = np.array(res.image_res.original).astype(np.float32) / 127.5 - 1.0
+            original = original.transpose([2, 0, 1])[None]
+            mask_ = read_image(res.mask_res.original, None, anchor=None, to_mask=True)
+            remained_mask = ~(mask_.image >= 0.5)
+            sampled = np.where(remained_mask, original, sampled.numpy())
+            sampled = torch.from_numpy(sampled)
+        return sampled
 
     def outpainting(
         self,
@@ -574,6 +596,7 @@ class DiffusionAPI(APIMixin):
         max_wh: int = 512,
         num_steps: Optional[int] = None,
         clip_output: bool = True,
+        keep_original: bool = False,
         callback: Optional[Callable[[Tensor], Tensor]] = None,
         verbose: bool = True,
         **kwargs: Any,
@@ -594,6 +617,7 @@ class DiffusionAPI(APIMixin):
             max_wh=max_wh,
             num_steps=num_steps,
             clip_output=clip_output,
+            keep_original=keep_original,
             callback=callback,
             verbose=verbose,
             **kwargs,
@@ -662,10 +686,16 @@ class DiffusionAPI(APIMixin):
             mask,
             max_wh,
             anchor,
-            lambda remained_mask, img: remained_mask * img,
-            lambda arr: torch.where(torch.from_numpy(arr), 1.0, -1.0),
+            lambda remained_mask, img: np.where(remained_mask, img, 0.0),
+            lambda bool_mask: torch.where(torch.from_numpy(bool_mask), 1.0, -1.0),
         )
         cond = torch.cat([res.remained_image_cond, res.mask_cond], dim=1)
+        size = tuple(
+            map(
+                lambda n: n * self.size_info.factor,
+                res.remained_image_cond.shape[-2:][::-1],
+            )
+        )
         # refine with img2img
         if refine_fidelity is not None:
             z = self._get_z(res.image_res.image)
@@ -682,11 +712,10 @@ class DiffusionAPI(APIMixin):
                 **kwargs,
             )
         # sampling
-        z = torch.randn_like(res.remained_image_cond)
         return self.sample(
             1,
             export_path,
-            z=z,
+            size=size,  # type: ignore
             original_size=res.image_res.original_size,
             alpha=res.image_res.alpha if alpha is None else alpha,
             cond=cond,
