@@ -4,32 +4,35 @@ import torch
 from tqdm import tqdm
 from typing import Any
 from typing import Optional
+from cftool.misc import print_info
+from cftool.misc import random_hash
+from cftool.array import to_device
+from cftool.array import save_images
 
-from ...interface import load
-from ...interface import pack
-from ...interface import pixel_cnn
-from ....trainer import Trainer
+from ...api import load
+from ...api import pack
+from ...api import pixel_cnn
+from ....data import TensorData
+from ....data import CVDataModule
+from ....protocol import ITrainer
 from ....constants import INPUT_KEY
 from ....constants import LABEL_KEY
-from ....constants import INFO_PREFIX
 from ....constants import ORIGINAL_LABEL_KEY
-from ....data.interface import TensorData
-from ....data.interface import CVDataModule
-from ....misc.toolkit import to_device
-from ....misc.toolkit import save_images
 from ....misc.toolkit import eval_context
 from ....misc.internal_.callbacks import ImageCallback
 from ....models.cv.vae.vector_quantized import VQVAE
 
 
-def register_callback(vqvae: VQVAE, num_classes: Optional[int]) -> None:
-    @ImageCallback.register("pixel_cnn")
+def register_callback(vqvae: VQVAE, num_classes: Optional[int]) -> str:
+    tmp_name = random_hash()
+
+    @ImageCallback.register(tmp_name)
     class _(ImageCallback):
         def __init__(self, num_keep: int = 25, num_interpolations: int = 16):
             super().__init__(num_keep)
             self.num_interpolations = num_interpolations
 
-        def log_artifacts(self, trainer: Trainer) -> None:
+        def log_artifacts(self, trainer: ITrainer) -> None:
             if not self.is_rank_0:
                 return None
             device = trainer.device
@@ -38,7 +41,7 @@ def register_callback(vqvae: VQVAE, num_classes: Optional[int]) -> None:
             original_indices = batch[INPUT_KEY]
             img_size = original_indices.shape[2]
             batch_size = original_indices.shape[0]
-            model = trainer.model
+            model = trainer.model.core
             with eval_context(model):
                 sampled_indices = model.sample(batch_size, img_size)
             with eval_context(vqvae):
@@ -78,8 +81,12 @@ def register_callback(vqvae: VQVAE, num_classes: Optional[int]) -> None:
                         os.path.join(conditional_folder, f"interpolation_{i}.png"),
                     )
 
+    return tmp_name
+
 
 class VQVAEInference:
+    tmp_callback_name: Optional[str] = None
+
     def __init__(
         self,
         workplace: str,
@@ -93,14 +100,17 @@ class VQVAEInference:
     ):
         self.cuda = cuda
         self.debug = kwargs.get("debug", False)
-        self.vqvae = load(pack(vqvae_log_folder), cuda=cuda).model
+        self.vqvae = load(pack(vqvae_log_folder), cuda=cuda).model.core
         self.code_export_folder = os.path.join(workplace, "codes")
-        register_callback(self.vqvae, num_classes)
+        if VQVAEInference.tmp_callback_name is not None:
+            ImageCallback.remove(VQVAEInference.tmp_callback_name)
+        VQVAEInference.tmp_callback_name = register_callback(self.vqvae, num_classes)
         if inference_model == "pixel_cnn":
             model_config = kwargs.setdefault("model_config", {})
             model_config["need_embedding"] = True
             model_config["num_conditional_classes"] = num_classes
             kwargs.setdefault("workplace", os.path.join(workplace, "_logs"))
+            kwargs.setdefault("callback_names", VQVAEInference.tmp_callback_name)
             self.m = pixel_cnn(num_codes, **kwargs)
         else:
             msg = f"inference model '{inference_model}' is not implemented"
@@ -111,8 +121,7 @@ class VQVAEInference:
         finished_path = os.path.join(export_folder, "__finished__")
         if os.path.isfile(finished_path):
             return None
-        data.prepare(None)
-        code_train, code_valid = data.initialize()
+        code_train, code_valid = data.get_loaders()
         for name, loader in zip(["train", "valid"], [code_train, code_valid]):
             if loader is None:
                 continue
@@ -126,11 +135,11 @@ class VQVAEInference:
                     break
             all_codes = torch.cat(code_indices, dim=0)
             path = os.path.join(export_folder, f"{name}.pt")
-            print(f"{INFO_PREFIX}saving {name} codes ({all_codes.shape})")
+            print_info(f"saving {name} codes ({all_codes.shape})")
             torch.save(all_codes, path)
             all_labels = torch.cat(labels, dim=0)
             label_path = os.path.join(export_folder, f"{name}_labels.pt")
-            print(f"{INFO_PREFIX}saving {name} labels ({all_labels.shape})")
+            print_info(f"saving {name} labels ({all_labels.shape})")
             torch.save(all_labels, label_path)
         if not self.debug:
             with open(finished_path, "w"):
