@@ -1,28 +1,25 @@
-import torch
-
+from torch import nn
 from torch import Tensor
 from typing import Any
 from typing import Dict
 from typing import Optional
-from cftool.misc import check_requires
+from cftool.types import tensor_dict_type
 
 from .vanilla import reparameterize
-from ..protocol import GaussianGeneratorMixin
-from ....types import tensor_dict_type
-from ....protocol import TrainerState
-from ....protocol import ModelProtocol
 from ....constants import LABEL_KEY
 from ....constants import PREDICTIONS_KEY
-from ..encoder.protocol import Encoder1DBase
+from ..encoder.protocol import make_encoder
 from ...implicit.siren import ImgSiren
+from ...protocols.cv import GaussianGeneratorMixin
 from ....misc.toolkit import auto_num_layers
 from ....losses.vae import MU_KEY
 from ....losses.vae import LOG_VAR_KEY
 from ....modules.blocks import Linear
+from ....misc.internal_.register import register_module
 
 
-@ModelProtocol.register("siren_vae")
-class SirenVAE(ModelProtocol, GaussianGeneratorMixin):
+@register_module("siren_vae")
+class SirenVAE(nn.Module, GaussianGeneratorMixin):
     def __init__(
         self,
         img_size: int,
@@ -50,16 +47,13 @@ class SirenVAE(ModelProtocol, GaussianGeneratorMixin):
         args = img_size, min_size, target_downsample
         num_downsample = auto_num_layers(*args, use_stride=encoder1d == "vanilla")
         # encoder
-        encoder1d_base = Encoder1DBase.get(encoder1d)
         if encoder1d_config is None:
             encoder1d_config = {}
-        if check_requires(encoder1d_base, "img_size"):
-            encoder1d_config["img_size"] = img_size
-        encoder1d_config["in_channels"] = in_channels
-        encoder1d_config["latent_dim"] = latent_dim
-        if encoder1d == "vanilla":
-            encoder1d_config["num_downsample"] = num_downsample
-        self.encoder = encoder1d_base(**encoder1d_config)
+        encoder1d_config.setdefault("img_size", img_size)
+        encoder1d_config.setdefault("in_channels", in_channels)
+        encoder1d_config.setdefault("latent_dim", latent_dim)
+        encoder1d_config.setdefault("num_downsample", num_downsample)
+        self.encoder = make_encoder(encoder1d, encoder1d_config, is_1d=True)
         self.to_statistics = Linear(latent_dim, 2 * latent_dim, bias=False)
         # siren
         self.siren = ImgSiren(
@@ -87,17 +81,9 @@ class SirenVAE(ModelProtocol, GaussianGeneratorMixin):
         size: Optional[int] = None,
         **kwargs: Any,
     ) -> Tensor:
-        net = self.siren.decode(z, labels=labels, size=size)
-        net = torch.tanh(net)
-        return net
+        return self.siren.decode(z, labels=labels, size=size)
 
-    def forward(
-        self,
-        batch_idx: int,
-        batch: tensor_dict_type,
-        state: Optional[TrainerState] = None,
-        **kwargs: Any,
-    ) -> tensor_dict_type:
+    def forward(self, batch: tensor_dict_type, **kwargs: Any) -> tensor_dict_type:
         net = self.encoder.encode(batch, **kwargs)
         net = self.to_statistics(net)
         mu, log_var = net.chunk(2, dim=1)
