@@ -1,9 +1,9 @@
 from typing import Any
 from typing import Optional
 from cftool.array import l2_normalize
+from cftool.types import tensor_dict_type
 
 from .core import HuggingFaceModel
-from ....types import tensor_dict_type
 from ....protocol import TrainerState
 from ....constants import LATENT_KEY
 
