@@ -166,10 +166,10 @@ class PositionalEncoding(Module):
     def __init__(
         self,
         dim: int,
-        num_history: int,
+        num_tokens: int,
         dropout: float = 0.0,
         *,
-        num_heads: int,
+        num_head_tokens: int,
         is_vision: bool,
         enable: bool = True,
     ):
@@ -178,9 +178,9 @@ class PositionalEncoding(Module):
         self.pos_encoding = None
         if enable:
             self.pos_drop = nn.Dropout(p=dropout)
-            self.pos_encoding = nn.Parameter(torch.zeros(1, num_history, dim))
+            self.pos_encoding = nn.Parameter(torch.zeros(1, num_tokens, dim))
             nn.init.trunc_normal_(self.pos_encoding, std=0.02)
-        self.num_heads = num_heads
+        self.num_head_tokens = num_head_tokens
         self.is_vision = is_vision
 
     def forward(
@@ -192,11 +192,17 @@ class PositionalEncoding(Module):
         if self.pos_encoding is None or self.pos_drop is None:
             return net
         if self.is_vision:
+            feature_span = 0
             pos_encoding = self.interpolate_pos_encoding(net, hwp)
         else:
-            pos_encoding = self.pos_encoding[:, : net.shape[1] - self.num_heads]
+            feature_span = net.shape[1] - self.num_head_tokens
+            pos_encoding = self.pos_encoding[:, :feature_span]
         pos_encoding = self.pos_drop(pos_encoding)
-        return net + pos_encoding
+        if self.is_vision or self.num_head_tokens <= 0:
+            return net + pos_encoding
+        net = net.clone()
+        net[:, :feature_span] += pos_encoding
+        return net
 
     # this is for vision positional encodings
     def interpolate_pos_encoding(
@@ -206,8 +212,8 @@ class PositionalEncoding(Module):
     ) -> Tensor:
         pos_encoding = self.pos_encoding
         assert pos_encoding is not None
-        num_current_history = net.shape[1] - self.num_heads
-        num_history = pos_encoding.shape[1] - self.num_heads
+        num_current_history = net.shape[1] - self.num_head_tokens
+        num_history = pos_encoding.shape[1] - self.num_head_tokens
         if hwp is None:
             w = h = patch_size = None
         else:
@@ -217,9 +223,9 @@ class PositionalEncoding(Module):
         if w is None or h is None or patch_size is None:
             raise ValueError("`hwp` should be provided for `interpolate_pos_encoding`")
         head_encoding = None
-        if self.num_heads > 0:
-            head_encoding = pos_encoding[:, : self.num_heads]
-            pos_encoding = pos_encoding[:, self.num_heads :]
+        if self.num_head_tokens > 0:
+            head_encoding = pos_encoding[:, : self.num_head_tokens]
+            pos_encoding = pos_encoding[:, self.num_head_tokens :]
         dim = net.shape[-1]
         # This assume that the original input is squared image
         sqrt = math.sqrt(num_history)
@@ -242,7 +248,7 @@ class MixedStackedEncoder(Module):
     def __init__(
         self,
         dim: int,
-        num_history: int,
+        num_tokens: int,
         *,
         token_mixing_type: str,
         token_mixing_config: Optional[Dict[str, Any]] = None,
@@ -272,28 +278,28 @@ class MixedStackedEncoder(Module):
         # head token
         self.aux_heads = aux_heads
         if not use_head_token:
-            num_heads = 0
+            num_head_tokens = 0
             self.head_token = None
         else:
-            num_heads = 1
+            num_head_tokens = 1
             if aux_heads is not None:
-                num_heads += len(aux_heads)
-            self.head_token = nn.Parameter(torch.zeros(1, num_heads, dim))
-        self.num_heads = num_heads
+                num_head_tokens += len(aux_heads)
+            self.head_token = nn.Parameter(torch.zeros(1, num_head_tokens, dim))
+        self.num_heads = num_head_tokens
         # positional encoding
-        num_history += num_heads
+        num_tokens += num_head_tokens
         if is_vision_positional_encoding is None:
             if use_positional_encoding:
                 raise ValueError(
                     "`is_vision_positional_encoding` should be specified when "
-                    "`is_vision_positional_encoding` is set to True"
+                    "`use_positional_encoding` is set to True"
                 )
             is_vision_positional_encoding = False
         self.pos_encoding = PositionalEncoding(
             dim,
-            num_history,
+            num_tokens,
             positional_encoding_dropout,
-            num_heads=num_heads,
+            num_head_tokens=num_head_tokens,
             is_vision=is_vision_positional_encoding,
             enable=use_positional_encoding,
         )
@@ -309,7 +315,7 @@ class MixedStackedEncoder(Module):
         self.mixing_blocks = ModuleList(
             [
                 MixingBlock(
-                    num_history,
+                    num_tokens,
                     dim,
                     feedforward_dim,
                     norm_position=norm_position,
@@ -373,10 +379,12 @@ class MixedStackedEncoder(Module):
         self.apply(self._init_weights)
 
     def _init_weights(self, m: Module) -> None:
-        if isinstance(m, nn.Linear) or isinstance(m, Linear):
+        if isinstance(m, nn.Linear):
             nn.init.trunc_normal_(m.weight, std=0.02)
             if m.bias is not None:
                 nn.init.constant_(m.bias, 0.0)
+        elif isinstance(m, Linear):
+            m.init_weights_with(lambda t: nn.init.trunc_normal_(t, std=0.02))
         elif isinstance(m, nn.LayerNorm):
             nn.init.constant_(m.bias, 0.0)
             nn.init.constant_(m.weight, 1.0)
